<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity(repositoryClass="App\Repository\UserRepository")
 */
class User {
    /**
     * @ORM\Id()
     * @ORM\GeneratedValue()
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=70)
     */
    private $email;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $pass;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $name;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $fistname;

    /**
     * @ORM\Column(type="smallint")
     */
    private $gender;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $avatar;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Language")
     */
    private $language;

    /**
     * @ORM\ManyToMany(targetEntity="App\Entity\User")
     */
    private $friends;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\ConfigNotification", mappedBy="user", orphanRemoval=true)
     */
    private $configNotifications;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\UserNotification", mappedBy="user", orphanRemoval=true)
     */
    private $notifications;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Wish", mappedBy="owner")
     */
    private $ownedWishes;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Participant", mappedBy="user", orphanRemoval=true)
     */
    private $participations;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Comment", mappedBy="user", orphanRemoval=true)
     */
    private $comments;

    public function __construct () {
        $this->friends = new ArrayCollection();
        $this->configNotifications = new ArrayCollection();
        $this->notifications = new ArrayCollection();
        $this->memberOfGroups = new ArrayCollection();
        $this->ownedWishes = new ArrayCollection();
        $this->participations = new ArrayCollection();
        $this->comments = new ArrayCollection();
    }

    public function getId (): ?int {
        return $this->id;
    }

    public function getEmail (): ?string {
        return $this->email;
    }

    public function setEmail (string $email): self {
        $this->email = $email;

        return $this;
    }

    public function getPass (): ?string {
        return $this->pass;
    }

    public function setPass (string $pass): self {
        $this->pass = $pass;

        return $this;
    }

    public function getName (): ?string {
        return $this->name;
    }

    public function setName (string $name): self {
        $this->name = $name;

        return $this;
    }

    public function getFistname (): ?string {
        return $this->fistname;
    }

    public function setFistname (?string $fistname): self {
        $this->fistname = $fistname;

        return $this;
    }

    public function getGender (): ?int {
        return $this->gender;
    }

    public function setGender (int $gender): self {
        $this->gender = $gender;

        return $this;
    }

    public function getAvatar (): ?string {
        return $this->avatar;
    }

    public function setAvatar (?string $avatar): self {
        $this->avatar = $avatar;

        return $this;
    }

    public function getLanguage (): ?Language {
        return $this->language;
    }

    public function setLanguage (?Language $language): self {
        $this->language = $language;

        return $this;
    }

    /**
     * @return Collection|self[]
     */
    public function getFriends (): Collection {
        return $this->friends;
    }

    public function addFriend (self $friend): self {
        if (!$this->friends->contains($friend)) {
            $this->friends[] = $friend;
        }

        return $this;
    }

    public function removeFriend (self $friend): self {
        if ($this->friends->contains($friend)) {
            $this->friends->removeElement($friend);
        }

        return $this;
    }

    /**
     * @return Collection|ConfigNotification[]
     */
    public function getConfigNotifications (): Collection {
        return $this->configNotifications;
    }

    public function addConfigNotification (ConfigNotification $configNotification): self {
        if (!$this->configNotifications->contains($configNotification)) {
            $this->configNotifications[] = $configNotification;
            $configNotification->setUser($this);
        }

        return $this;
    }

    public function removeConfigNotification (ConfigNotification $configNotification): self {
        if ($this->configNotifications->contains($configNotification)) {
            $this->configNotifications->removeElement($configNotification);
            // set the owning side to null (unless already changed)
            if ($configNotification->getUser() === $this) {
                $configNotification->setUser(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|UserNotification[]
     */
    public function getNotifications (): Collection {
        return $this->notifications;
    }

    public function addNotification (UserNotification $notification): self {
        if (!$this->notifications->contains($notification)) {
            $this->notifications[] = $notification;
            $notification->setUser($this);
        }

        return $this;
    }

    public function removeNotification (UserNotification $notification): self {
        if ($this->notifications->contains($notification)) {
            $this->notifications->removeElement($notification);
            // set the owning side to null (unless already changed)
            if ($notification->getUser() === $this) {
                $notification->setUser(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Wish[]
     */
    public function getOwnedWishes(): Collection
    {
        return $this->ownedWishes;
    }

    public function addOwnedWish(Wish $ownedWish): self
    {
        if (!$this->ownedWishes->contains($ownedWish)) {
            $this->ownedWishes[] = $ownedWish;
            $ownedWish->setOwner($this);
        }

        return $this;
    }

    public function removeOwnedWish(Wish $ownedWish): self
    {
        if ($this->ownedWishes->contains($ownedWish)) {
            $this->ownedWishes->removeElement($ownedWish);
            // set the owning side to null (unless already changed)
            if ($ownedWish->getOwner() === $this) {
                $ownedWish->setOwner(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Participant[]
     */
    public function getParticipations(): Collection
    {
        return $this->participations;
    }

    public function addParticipation(Participant $participation): self
    {
        if (!$this->participations->contains($participation)) {
            $this->participations[] = $participation;
            $participation->setUser($this);
        }

        return $this;
    }

    public function removeParticipation(Participant $participation): self
    {
        if ($this->participations->contains($participation)) {
            $this->participations->removeElement($participation);
            // set the owning side to null (unless already changed)
            if ($participation->getUser() === $this) {
                $participation->setUser(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Comment[]
     */
    public function getComments(): Collection
    {
        return $this->comments;
    }

    public function addComment(Comment $comment): self
    {
        if (!$this->comments->contains($comment)) {
            $this->comments[] = $comment;
            $comment->setUser($this);
        }

        return $this;
    }

    public function removeComment(Comment $comment): self
    {
        if ($this->comments->contains($comment)) {
            $this->comments->removeElement($comment);
            // set the owning side to null (unless already changed)
            if ($comment->getUser() === $this) {
                $comment->setUser(null);
            }
        }

        return $this;
    }
}
